import '/core/ui/options/options.js';
import { CategoryData, OptionType, ShowReloadUIPrompt, ShowRestartGamePrompt, CategoryType, GetGroupLocKey } from '/core/ui/options/options-helpers.js';
import { InputEngineEventName } from '/core/ui/input/input-support.js';
import { displayTypeOption, modSelectorOption, openOnTabOption } from '../modOptions.js';

export class ScreenOptionsDecorator {
    constructor(val) {
        this.screenOptions = val;

        this.groupClicked = {}
        this.groupHeaders = null;
        this.groupClicked = {}
        this.modCategoryPanel = null;

        this.settings = {
            lastOpenTab: null
        };
        this.loadSettings()
    }

    beforeAttach() {

        // Init openOnTabOption
        for (const tabData of this.screenOptions.tabData){
            if (openOnTabOption.dropdownItems.findIndex(tab => tab.value === tabData.id) == -1){
                openOnTabOption.dropdownItems.push({value: tabData.id, label: tabData.label})
            }
        }
    }

    loadSettings() {
        const modSettings = localStorage.getItem("modSettings");
        try {
            if (modSettings) {
                const modSetting = JSON.parse(modSettings)["ModOptionsManager"]
                if(modSetting){
                    this.settings = modSetting;
                }
            }
            else {
                console.warn("[ModOptionsManager] No settings found, using defaults");
            }
        }
        catch (e) {
            console.error("[ModOptionsManager] Error loading settings", e);
        }
    }

    saveSettings() {
        const modSettings = JSON.parse(localStorage.getItem("modSettings") || '{}');
        modSettings["ModOptionsManager"] = this.settings
        localStorage.setItem("modSettings", JSON.stringify(modSettings));
    }


    afterAttach() {
        // getGroup Header from sreenOptions
        this.groupHeaders = this.screenOptions.modCategoryPanel.component.getGroupHeaders()

        // select Tab based on Option
        var tabToOpen = ""
        if (openOnTabOption.selectedItemIndex == 0) {
            tabToOpen = this.settings.lastOpenTab
        } else {
            tabToOpen = openOnTabOption.value
        }

        for (let index = 0; index < this.screenOptions.tabData.length; ++index) {
            if (this.screenOptions.tabData[index].id == tabToOpen) {
                this.screenOptions.tabControl.setAttribute("selected-tab-index", index);
                break;
            }
        }

        if (this.screenOptions.modSelectorElement) {
            this.screenOptions.modSelectorElement.addEventListener('dropdown-selection-change', (event) => { this.onModCategoryUpdate(event.detail.selectedIndex)});
        }

        // MSM: loop through groupHeaders to add event listener
        for (const [group, header] of Object.entries(this.groupHeaders)) {
            header.addEventListener(InputEngineEventName, (inputEvent) => this.onGroupToggle(inputEvent, group));
        }

        // MSM: init rendering on first select to hide all other if displayTypeOption is combobox
        if (displayTypeOption.value == "combobox"){
            // calculate selectedItemIndex as List not here during OptionAPI init
            for (let index = 0; index < modSelectorOption.dropdownItems.length; ++index) {
                if (modSelectorOption.dropdownItems[index].value == modSelectorOption.value) {
                    modSelectorOption.selectedItemIndex = index;
                    break;
                }
            }
            modSelectorOption.forceRender();
            this.onModCategoryUpdate(modSelectorOption.selectedItemIndex);
        }
    }



    beforeDetach() {
        // MSM: loop through groupHeaders to remove event listener
        for (const [group, header] of Object.entries(this.groupHeaders)) {
            header?.removeEventListener('engine-input', (inputEvent) => this.onGroupToggle(inputEvent, group));
        }
    }

    afterDetach() {
        this.saveLastOpen()
    }

    saveLastOpen() {
        const selectedTab = this.screenOptions.tabData[this.screenOptions.tabControl.getAttribute("selected-tab-index")];
        this.settings.lastOpenTab = selectedTab.id;
        this.saveSettings();
    }

    onAttributeChanged(name, prev, next) {}

    onModCategoryUpdate = (selectedIndex) => {
        const modSelected = modSelectorOption.dropdownItems[selectedIndex].value;
        for(const option of this.screenOptions.modOptions){
            const modName = option.mod?.value ?? option.groupFlat;
            option.isHidden = (modName != modSelected);
            if (option.forceRender) {
                option.forceRender();
            }
            const group = this.screenOptions.Root.querySelector(`[data-group="${option.group}"]`);
            group.classList.toggle("hidden", (modName != modSelected));
        }
    }

    onGroupToggle = (inputEvent, group) =>{
        //if group was clicked less than 400ms ago, ignore the event. (inputEvent Detail a empty ...)
        if (this.groupClicked[group] && Date.now() - this.groupClicked[group] < 400){
            return
        }
        //store date time a group was clicked
        this.groupClicked[group] = Date.now()


        var divider =  this.screenOptions.Root.querySelector(`[data-group="${group}"] div.filigree-divider-h3`);
        var hidden = false
        if(divider){
            divider.classList.remove("filigree-divider-h3")
            divider.classList.add("filigree-shell-small")
            hidden = true
        } else {
            divider =  this.screenOptions.Root.querySelector(`[data-group="${group}"] div.filigree-shell-small`);
            divider.classList.remove("filigree-shell-small")
            divider.classList.add("filigree-divider-h3")
        }
        if (group == 'mod_selection'){
            modSelectorOption.isHidden = hidden;
            modSelectorOption.forceRender()
        }
        //toggle all options in group
        for(const option of this.screenOptions.modOptions){
            if (option.group == group){
                option.isHidden = hidden;
                if (option.forceRender) {
                    option.forceRender();
                }
            }
        }
    }
}

Controls.decorate('screen-options', (val) => new ScreenOptionsDecorator(val));